(**********************************************************************)
(*                                                                    *)
(*                                                                    *)
(* The OCaml archiver                                                 *)
(*                                                                    *)
(*        written by Gerd Stolpmann                                   *)
(*        Release 0.1                                                 *)
(*                                                                    *)
(*        This is free software, see LICENSE for details.             *)
(*                                                                    *)
(*                                                                    *)
(**********************************************************************)

(* Generates the part of the Makefile which compiles and links bytecode
 * programs.
 *)

(* TODO: Write it in a way such that Make_ocaml_native can inherit *)

class cls n as self =
  inherit Make_module.cls n

  method name () = "make_ocaml_bytecode"

  method enabled () = true

  method init_rules () =
    Make_module.rules_from_file (self # name() ^ ".init.pattern")

  method sterile_cmds files =
    (* Remove simply all <files>. *)
    Make_module.gen_sterile_cmds 
      (fun f -> [f])
      files

  method clean_cmds files goals =
    Make_module.gen_sterile_cmds
      (Make_module.gen_clean_filter ["indep"] goals)
      files

  method generate_rules () =
    []

  method dependencies_cmds () =
    []

  method dependencies_obj () =
    None

  method compile_rules () =
    Make_module.rules_from_file (self # name() ^ ".compile.pattern")

  method link_rules () =
    (* 1. find the required libs             --> link_libs
     * 2. select the libs with useful type   --> link_libs
     * 3. find all link rules in Ocamlmake
     * 4. For each link rule:
     *    a. determine the dependency
     *    b. determine the linker. drop if linker isnt ocamlc or ocamlmktop
     *    c. find out if archive has to be made
     *    concatenate 
     *)
    ...

  method link_libs () : (string * string * string) list =
    (* Find the required libraries and select those with useful type.
     * Returns list of triples (node_name, dir_name, file_name), where
     * the library belongs to node <node_name>, resides in the directory
     * <dir_name> relative to the node and has the name <file_name>.
     * A library is "useful" if it has type "make_ocaml_bytecode" or
     * "make_ocaml_c" and
     * has one of the suffixes ".cma", ".cmo", ".o" or ".a". 
     *)
    let reqs = Node.requirements node in

  method linked_files (objects : string list) : (string list * bool) =
    (* Expands "<libs>" in <objects, removes "<toploop>" and returns a
     * list of files, either absoulte file names or relative ones.
     * returns (list_of_files, includes_toploop?)
     *)
    ...

  method which_linker object_files includes_toploop =
    (* Analyzes the name of the <object_files> and returns either:
     * "ocamlc", "ocamlopt" or "ocamlmktop" or fails.
     * Use the result of linked_files as <object_files>
     *)
    ...

  method custom_mode object_files includes_toploop =
    (* Returns true if "-custom" is needed *)
    ...

  method archive_mode goal linker =
    (* Returns true if "-a" is needed. <goal> is the name of the 
     * goal. <linker> is the result of which_linker.
     *)
    ...

  method link_objs () =
    ...
end
