(**********************************************************************)
(*                                                                    *)
(*                                                                    *)
(* The OCaml archiver                                                 *)
(*                                                                    *)
(*        written by Gerd Stolpmann                                   *)
(*        Release 0.1                                                 *)
(*                                                                    *)
(*        This is free software, see LICENSE for details.             *)
(*                                                                    *)
(*                                                                    *)
(**********************************************************************)

(* Generates the part of the Makefile which calls ocaml code generators
 * such as ocamllex and ocamlyacc.
 * Furthermore, the dependencies of the ALL source code files are
 * discovered.
 * In general, this class does all what is necessary for both bytecode
 * and native compilation passes.
 *)

class cls n =

  inherit Make_module.cls n

  method name () = "make_ocaml_code"

  method enabled () = true

  method init_rules () =
    Make_module.rules_from_file "make_ocaml_code.init.pattern"

  method sterile_cmds files =
    (* Remove simply all <files>. *)
    Make_module.gen_sterile_cmds 
      (fun f -> [f])
      files

  method clean_cmds files goals =
    (* Well, it is relative senseless to have a goal of this type;
     * perhaps debugging would be an application.
     * The directory in which the goals are is "gen".
     *)
    Make_module.gen_sterile_cmds
      (Make_module.gen_clean_filter ["gen"] goals)
      files

  method generate_rules () =
    Make_module.rules_from_file "make_ocaml_code.generate.pattern"

  method dependencies_cmds () =
    Make_module.rules_from_file "make_ocaml_code.dependencies.cmds"

  method dependencies_obj () =
    Some "Dependencies.ocaml_code"

  method compile_rules () =
    []

  method link_rules () =
    []

  method link_objs () =
    []
end
